@@ -114,22 +114,32 @@ convertNative <- function(unit, dimension="x", type="location") {
   .Defunct("convertUnit")
 }
 
-deviceLoc <- function(x, y, valueOnly=FALSE) {
-    result <- grid.Call(C_devLoc, x, y)
+deviceLoc <- function(x, y, valueOnly=FALSE, device=FALSE) {
+    result <- grid.Call(C_devLoc, x, y, as.logical(device))
     names(result) <- c("x", "y")
-    if (!valueOnly)
-        list(x=unit(result$x, "in"), y=unit(result$y, "in"))
-    else
+    if (valueOnly) {
         result
+    } else {
+        if (device) {
+            list(x=unit(result$x, "native"), y=unit(result$y, "native"))
+        } else {
+            list(x=unit(result$x, "in"), y=unit(result$y, "in"))
+        }
+    }
 }
 
-deviceDim <- function(w, h, valueOnly=FALSE) {
-    result <- grid.Call(C_devDim, w, h)
+deviceDim <- function(w, h, valueOnly=FALSE, device=FALSE) {
+    result <- grid.Call(C_devDim, w, h, as.logical(device))
     names(result) <- c("w", "h")
-    if (!valueOnly)
-        list(w=unit(result$w, "in"), h=unit(result$h, "in"))
-    else
+    if (valueOnly) {
         result
+    } else {
+        if (device) {
+            list(w=unit(result$w, "native"), h=unit(result$h, "native"))
+        } else {
+            list(w=unit(result$w, "in"), h=unit(result$h, "in"))
+        }
+    }
 }
 
 # This is like the "convert" functions:  it evaluates units (immediately)
@@ -311,6 +321,7 @@ Summary.unit <- function(..., na.rm=FALSE) {
     if (nMatches != 0) {
         data <- lapply(x, .subset2, 2L)
         amount <- vapply(x, .subset2, numeric(1), 1L)[matchUnits]
+        amount <- rep(amount, lengths(data[matchUnits]))
         matchData <- unclass(unlist(data[matchUnits], recursive = FALSE))
         for (i in seq_along(amount)) {
             if (amount[i] != 1) 
