@@ -1,7 +1,7 @@
 #  File src/library/tools/R/Rd2pdf.R
 #  Part of the R package, https://www.R-project.org
 #
-#  Copyright (C) 1995-2015 The R Core Team
+#  Copyright (C) 1995-2021 The R Core Team
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -19,17 +19,44 @@
 #### R based engine for  R CMD Rdconv|Rd2pdf
 ####
 
-##' @param args
-
-##' @return ...
-
 ## base packages do not have versions and this is called on
 ## DESCRIPTION.in
 ## encodings are tricky: this may be done in a foreign encoding
 ## (e.g., Latin-1 in UTF-8)
 .DESCRIPTION_to_latex <- function(descfile, outfile, version = "Unknown")
 {
-    desc <- read.dcf(descfile)[1, ]
+    mygsub <- function(...) {
+        .gsub_with_transformed_matches(..., useBytes = TRUE)
+    }
+    texify <- function(x, one = TRUE, two = FALSE) {
+        ## Handle LaTeX special characters.
+        ## one: handle # $ % & _ ^ ~
+        ##      backslash escape the first five
+        ##      replace ^ by \textasciicircum{}
+        ##      replace ~ by \textasciitilde{}
+        ## two: handle { } \
+        ##      backslash escape the first two
+        ##      replace \ by \textbackslash{}
+        if(two)
+            x <- fsub("\\", "\\textbackslash", x)
+        if(one) {
+            x <- psub("([#$%&_])", "\\\\\\1", x)
+            x <- fsub("^", "\\textasciicircum", x)
+            x <- fsub("~", "\\textasciitilde", x)
+        }
+        if(two) {
+            x <- psub("([{}])", "\\\\\\1", x)
+            x <- fsub("\\textbackslash", "\\textbackslash{}", x)
+        }
+        if(one) {
+            x <- fsub("\\textasciicircum", "\\textasciicircum{}", x)
+            x <- fsub("\\textasciitilde", "\\textasciitilde{}", x)
+        }
+        x
+    }
+
+
+    desc <- read.dcf(descfile)[1L, ]
     ## Using
     ##   desc <- .read_description(descfile)
     ## would preserve leading white space in Description and Author ...
@@ -37,12 +64,35 @@
         out <- file(outfile, "a")
         on.exit(close(out))
     } else out <- outfile
-    cat("\\begin{description}", "\\raggedright{}", sep="\n", file=out)
     fields <- names(desc)
-    fields <- fields[! fields %in% c("Package", "Packaged", "Built")]
+    fields <- fields %w/o% c("Package", "Packaged", "Built")
     if ("Encoding" %in% fields)
         cat("\\inputencoding{", latex_canonical_encoding(desc["Encoding"]),
             "}\n", sep = "", file = out)
+    ## Also try adding PDF title and author metadata.
+    tit <- desc["Title"]
+    tit <- paste0(desc["Package"], ": ",
+                  texify(gsub("[[:space:]]+", " ", tit), two = TRUE))
+    cat(paste0("\\ifthenelse{\\boolean{Rd@use@hyper}}",
+               "{\\hypersetup{pdftitle = {", tit, "}}}{}"),
+        file = out)
+    ## Only try author from Authors@R.
+    if(!is.na(aar <- desc["Authors@R"])) {
+        aar <- tryCatch(utils:::.read_authors_at_R_field(aar),
+                        error = identity)
+        if(!inherits(aar, "error")) {
+            aar <- Filter(utils:::.person_has_author_role, aar)
+            aut <- format(aar, include = c("given", "family"))
+            aut <- paste(aut[nzchar(aut)], collapse = "; ")
+            aut <- texify(gsub("[[:space:]]+", " ", aut), two = TRUE)
+            if(nzchar(aut))
+                cat(paste0("\\ifthenelse{\\boolean{Rd@use@hyper}}",
+                           "{\\hypersetup{pdfauthor = {", aut, "}}}{}"),
+                    file = out)
+        }
+    }
+    ## And now the actual content.
+    cat("\\begin{description}", "\\raggedright{}", sep="\n", file=out)    
     for (f in fields) {
         ## Drop 'Authors@R' for now: this is formatted badly by \AsIs,
         ## and ideally was used for auto-generating the Author and
@@ -56,26 +106,43 @@
         ## \Rd@AsIs@dospecials in Rd.sty handles the first seven, so
         ## braces and backslashes need explicit handling.
         text <- gsub('"([^"]*)"', "\\`\\`\\1''", text, useBytes = TRUE)
-        text <- gsub("\\", "\\textbackslash{}", text,
-                     fixed = TRUE, useBytes = TRUE)
-        text <- gsub("([{}$#_])", "\\\\\\1", text, useBytes = TRUE)
-        text <- gsub("@VERSION@", version, text, fixed = TRUE, useBytes = TRUE)
+        text <- texify(text, one = FALSE, two = TRUE)
+        text <- fsub("@VERSION@", version, text)
         ## text can have paras, and digest/DESCRIPTION does.
         ## \AsIs is per-para.
         text <- strsplit(text, "\n\n", fixed = TRUE, useBytes = TRUE)[[1L]]
         Encoding(text) <- "unknown"
         if(f %in% c("Author", "Maintainer", "Contact"))
-            text <- gsub("<([^@ ]+)@([^> ]+)>",
-                         "}\\\\email{\\1@\\2}\\\\AsIs{",
-                         text, useBytes = TRUE)
-        if(f %in% c("URL", "BugReports", "Contact"))
+            text <- mygsub("<([^@ ]+)@([^> ]+)>",
+                           "}\\\\email{%s@%s}\\\\AsIs{",
+                           text,
+                           list(texify, texify),
+                           c(1L, 2L))
+        if(f %in% c("URL", "BugReports", "Additional_repositories"))
             text <- gsub("(http://|ftp://|https://)([^[:space:],]+)",
                          "}\\\\url{\\1\\2}\\\\AsIs{",
                          text, useBytes = TRUE)
+        if(f %in% c("Author",       # possibly with ORCID URLs inside <>
+                    "Description")) {
+            text <- gsub("<(http://|ftp://|https://)([^[:space:],>]+)>",
+                         "<}\\\\url{\\1\\2}\\\\AsIs{>",
+                         text, useBytes = TRUE)
+        }
+        if(f == "Description") {   # DOI and arXiv identifiers inside <>
+            text <- mygsub("<(DOI:|doi:)([[:space:]]*)([^[:space:]]+)>",
+                           "<}\\\\Rhref{https://doi.org/%s}{\\1%s}\\\\AsIs{>",
+                           text,
+                           list(identity, texify),
+                           c(3L, 3L))
+            ## Fancy escaping should not be needed for arXiv ids.
+            text <- gsub("<(arXiv:|arxiv:)([[:alnum:]/.-]+)([[:space:]]*\\[[^]]+\\])?>",
+                         "<}\\\\Rhref{https://arxiv.org/abs/\\2}{\\1\\2}\\\\AsIs{\\3>",
+                         text, useBytes = TRUE)
+        }
         text <- paste0("\\AsIs{", text, "}")
-        ## Not entirely safe: in theory, tags could contain \ ~ ^.
-        cat("\\item[", gsub("([#$%&_{}])", "\\\\\\1", f),
-            "]", paste(text, collapse = "\n\n"),  "\n", sep = "", file=out)
+        writeLines(paste0("\\item[", texify(f, TRUE, TRUE), "]",
+                          paste(text, collapse = "\n\n")),
+                   con = out, useBytes = TRUE)
     }
     cat("\\end{description}\n", file = out)
 }
@@ -120,9 +187,12 @@
                                 attr(res,"latexEncoding"))
             lines <- readLines(out)
             if (attr(res, "hasFigures")) {
-                graphicspath <- paste("\\graphicspath{{",
-                                      normalizePath(file.path(dirname(f), "figures"), "/"),
-                                      "/}}", sep="")
+                graphicspath <-
+                    paste0("\\graphicspath{{\"",
+                           normalizePath(file.path(dirname(f),
+                                                   "figures"),
+                                         "/"),
+                           "/\"}}")
             	lines <- c(graphicspath, lines)
             	hasFigures <- TRUE
             }
@@ -141,6 +211,14 @@
              extraDirs = NULL, append = FALSE, silent = FALSE,
              pkglist = NULL)
 {
+    ## For Rd \packageFOO macro expansion:
+    path <- normalizePath(pkgdir)
+    if(file.exists(file.path(path, "DESCRIPTION")))
+        Sys.setenv("_R_RD_MACROS_PACKAGE_DIR_" = path)
+    else if((basename(path) == "man") &&
+            file.exists(file.path(dirname(path), "DESCRIPTION")))
+        Sys.setenv("_R_RD_MACROS_PACKAGE_DIR_" = dirname(path))
+
     ## sort order for topics, a little tricky
     re <- function(x) x[order(toupper(x), x)]
 
@@ -191,9 +269,12 @@
                                     attr(res, "latexEncoding"))
                 if (attr(res, "hasFigures")) {
                     lines <- readLines(outfilename)
-                    graphicspath <- paste("\\graphicspath{{",
-                    		    normalizePath(file.path(pkgdir, "help", "figures"), "/"),
-                    		    "/}}", sep="")
+                    graphicspath <-
+                        paste0("\\graphicspath{{\"",
+                               normalizePath(file.path(pkgdir, "help",
+                                                       "figures"),
+                                             "/"),
+                               "/\"}}")
                     writeLines(c(graphicspath, lines), outfilename)
                     hasFigures <- TRUE
                 }
@@ -238,7 +319,9 @@
             if (!silent) message("Converting Rd files to LaTeX ",
                                  appendLF = FALSE, domain = NA)
             cnt <- 0L
-            macros <- loadPkgRdMacros(pkgdir)
+            macros <- loadPkgRdMacros(pkgdir, initialRdMacros())
+            ## (Be nice and give the system macros also when 'pkgdir' is
+            ## not a package root directory.)
             macros <- initialRdMacros(pkglist, macros)
             for(i in seq_along(paths)) {
                 cnt <- cnt + 1L
@@ -256,12 +339,11 @@
                 if (attr(res, "hasFigures")) {
                     lines <- readLines(outfilename)
                     graphicspath <-
-                        paste("\\graphicspath{{",
-                              normalizePath(file.path(dirname(paths[i]),
-                                                      "figures"),
-                                            "/"),
-                              "/}}",
-                              sep = "")
+                        paste0("\\graphicspath{{\"",
+                               normalizePath(file.path(dirname(paths[i]),
+                                                       "figures"),
+                                             "/"),
+                               "/\"}}")
                     writeLines(c(graphicspath, lines), outfilename)
                     hasFigures <- TRUE
                 }
@@ -358,7 +440,7 @@
             "Using '-o \"\"' will choose an output filename by removing a '.Rd'",
             "extension from FILE and adding a suitable extension.",
             "",
-            "Report bugs at bugs.r-project.org .", sep = "\n")
+            "Report bugs at <https://bugs.R-project.org>.", sep = "\n")
     }
 
     options(showErrorCalls = FALSE, warn = 1)
@@ -388,7 +470,7 @@
                 R.version[["major"]], ".",  R.version[["minor"]],
                 " (r", R.version[["svn rev"]], ")\n", sep = "")
             cat("",
-                "Copyright (C) 1997-2009 The R Core Team.",
+                "Copyright (C) 1997-2015 The R Core Team.",
                 "This is free software; see the GNU General Public License version 2",
                 "or later for copying conditions.  There is NO warranty.",
                 sep="\n")
@@ -411,7 +493,7 @@
             os <- substr(a, 6, 1000)
         } else if (substr(a, 1, 11) == "--RdMacros=") {
             pkglist <- substr(a, 12, 1000)
-        } else if (substr(a, 1, 1) == "-") {
+        } else if (startsWith(a, "-")) {
             message("Warning: unknown option ", sQuote(a))
         } else files <- c(files, a)
         args <- args[-1L]
@@ -437,7 +519,7 @@
                if (!nzchar(enc)) enc <- "UTF-8"
                Rd2HTML(files, out, package = pkg, defines = os,
                        outputEncoding = enc, no_links = TRUE,
-                       stages = c("build", "install", "render"), 
+                       stages = c("build", "install", "render"),
                        macros = macros)
            },
            "latex" = {
@@ -497,9 +579,9 @@ function(pkgdir, outfile, title, batch = FALSE,
     ## trailer is for detection if we want to edit it later.
     latex_outputEncoding <- latex_canonical_encoding(outputEncoding)
     setEncoding <-
-        paste("\\usepackage[",
-              latex_outputEncoding, "]{",
-              inputenc, "} % @SET ENCODING@", sep="")
+        paste0("\\usepackage[",
+               latex_outputEncoding, "]{",
+               inputenc, "} % @SET ENCODING@")
     useGraphicx <- "% \\usepackage{graphicx} % @USE GRAPHICX@"
     writeLines(c(setEncoding,
                  if (inputenc == "inputenx" &&
@@ -591,15 +673,13 @@ function(pkgdir, outfile, title, batch = FALSE,
 		paste0("\\usepackage[", encs, "]{", inputenc, "}")
 	} else {
 	    setEncoding2 <-
-		paste(
-"\\usepackage[", encs, "]{", inputenc, "}
-\\IfFileExists{t2aenc.def}{\\usepackage[T2A]{fontenc}}{}", sep = "")
+		paste0("\\usepackage[", encs, "]{", inputenc, "}\n",
+                       "\\IfFileExists{t2aenc.def}{\\usepackage[T2A]{fontenc}}{}")
 	}
 	if (moreUnicode) {
 	    setEncoding2 <-
-		paste0(
-setEncoding2, "
-\\IfFileExists{ix-utf8enc.dfu}{\\input{ix-utf8enc.dfu}}{}")
+		paste0(setEncoding2,
+                       "\n\\IfFileExists{ix-utf8enc.dfu}{\\input{ix-utf8enc.dfu}}{}")
         }
         lines[lines == setEncoding] <- setEncoding2
 	if (hasFigures)
@@ -660,7 +740,7 @@ setEncoding2, "
             setwd(startdir)
             unlink(build_dir, recursive = TRUE)
         } else {
-            cat("You may want to clean up by 'rm -rf ", build_dir, "'\n", sep="")
+            cat("You may want to clean up by 'rm -Rf ", build_dir, "'\n", sep="")
         }
     }
 
@@ -707,7 +787,7 @@ setEncoding2, "
             "The output papersize is set by the environment variable R_PAPERSIZE.",
             "The PDF previewer is set by the environment variable R_PDFVIEWER.",
             "",
-            "Report bugs at bugs.r-project.org .",
+            "Report bugs at <https://bugs.R-project.org>.",
             sep = "\n")
     }
 
@@ -778,7 +858,7 @@ setEncoding2, "
             force <- TRUE
         } else if (a == "--only-meta") {
             only_meta <- TRUE
-        } else if (substr(a, 1, 5) == "--OS=" || substr(a, 1, 5) == "--OS=") {
+        } else if (substr(a, 1, 5) %in% c("--os=", "--OS=")) {
             OSdir <- substr(a, 6, 1000)
         } else if (substr(a, 1, 11) == "--encoding=") {
             enc <- substr(a, 12, 1000)
@@ -794,7 +874,7 @@ setEncoding2, "
             internals <- TRUE
         } else if (substr(a, 1, 11) == "--RdMacros=") {
             pkglist <- substr(a, 12, 1000)
-        } else if (substr(a, 1, 1) == "-") {
+        } else if (startsWith(a, "-")) {
             message("Warning: unknown option ", sQuote(a))
         } else files <- c(files, a)
         args <- args[-1L]
@@ -837,7 +917,7 @@ setEncoding2, "
         q("no", status = 2L, runLast = FALSE)
     }
     dir.create(build_dir, FALSE)
-    if(!nzchar(output)) output <- paste("Rd2", out_ext, sep = ".")
+    if(!nzchar(output)) output <- paste0("Rd2.", out_ext)
     if(file.exists(output) && !force) {
         cat("file", sQuote(output), "exists; please remove it first\n")
         q("no", status = 1L, runLast = FALSE)
@@ -866,7 +946,7 @@ setEncoding2, "
 
     setwd(startdir)
     cat("Saving output to", sQuote(output), "...\n")
-    file.copy(file.path(build_dir, paste("Rd2", out_ext, sep = ".")), output,
+    file.copy(file.path(build_dir, paste0("Rd2.", out_ext)), output,
               overwrite = force)
     cat("Done\n")
 
