@@ -1,6 +1,6 @@
 /*
  *  R : A Computer Language for Statistical Data Analysis
- *  Copyright (C) 2000-2019   The R Core Team.
+ *  Copyright (C) 2000-2020   The R Core Team.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -102,6 +102,7 @@ static Rboolean url_open(Rconnection con)
     void *ctxt;
     char *url = con->description;
     UrlScheme type = ((Rurlconn)(con->private))->type;
+    int mlen;
 
     if(con->mode[0] != 'r') {
 	REprintf("can only open URLs for reading");
@@ -159,7 +160,8 @@ static Rboolean url_open(Rconnection con)
     con->isopen = TRUE;
     con->canwrite = (con->mode[0] == 'w' || con->mode[0] == 'a');
     con->canread = !con->canwrite;
-    if(strlen(con->mode) >= 2 && con->mode[1] == 'b') con->text = FALSE;
+    mlen = (int) strlen(con->mode);
+    if(mlen >= 2 && con->mode[mlen - 1] == 'b') con->text = FALSE;
     else con->text = TRUE;
     con->save = -1000;
     set_iconv(con);
@@ -233,6 +235,7 @@ static Rboolean url_open2(Rconnection con)
     void *ctxt;
     char *url = con->description;
     UrlScheme type = ((Rurlconn)(con->private))->type;
+    int mlen;
 
     if(con->mode[0] != 'r') {
 	REprintf("can only open URLs for reading");
@@ -283,7 +286,8 @@ static Rboolean url_open2(Rconnection con)
     con->isopen = TRUE;
     con->canwrite = (con->mode[0] == 'w' || con->mode[0] == 'a');
     con->canread = !con->canwrite;
-    if(strlen(con->mode) >= 2 && con->mode[1] == 'b') con->text = FALSE;
+    mlen = (int) strlen(con->mode);
+    if(mlen >= 2 && con->mode[mlen - 1] == 'b') con->text = FALSE;
     else con->text = TRUE;
     con->save = -1000;
     set_iconv(con);
@@ -508,7 +512,7 @@ static SEXP in_do_download(SEXP args)
 	FILE *in, *out;
 	static char buf[CPBUFSIZE];
 	size_t n;
-	int nh = 7;
+	int nh = 7, mlen;
 #ifdef Win32
 	/* on Windows we have file:///d:/path/to
 	   whereas on Unix it is file:///path/to */
@@ -516,7 +520,9 @@ static SEXP in_do_download(SEXP args)
 #endif
 
 	/* Use binary transfers? */
-	in = R_fopen(R_ExpandFileName(url+nh), (mode[2] == 'b') ? "rb" : "r");
+	mlen = (int) strlen(mode);
+	in = R_fopen(R_ExpandFileName(url+nh),
+	             (mlen >= 2 && mode[mlen - 1] == 'b') ? "rb" : "r");
 	if(!in) {
 	    error(_("cannot open URL '%s', reason '%s'"),
 		  url, strerror(errno));
@@ -937,7 +943,7 @@ static void *in_R_HTTPOpen2(const char *url, const char *agent, const char *head
 
     // use keep-alive semantics, do not use local WinINet cache.
     DWORD flags = INTERNET_FLAG_KEEP_CONNECTION | INTERNET_FLAG_NO_CACHE_WRITE |
-                  INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTPS;
+                  INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTPS | INTERNET_FLAG_IGNORE_REDIRECT_TO_HTTP;
     if(!cacheOK) flags |= INTERNET_FLAG_PRAGMA_NOCACHE;
     wictxt->session = InternetOpenUrl(wictxt->hand, url, headers, headers ? -1 : 0, flags, 0);
     if(!wictxt->session) {
@@ -993,7 +999,7 @@ static void *in_R_HTTPOpen2(const char *url, const char *agent, const char *head
 		      HTTP_QUERY_CONTENT_LENGTH | HTTP_QUERY_FLAG_NUMBER,
 		      &len, &d1, &d2))
 	wictxt->length = len;
-    wictxt->type = strdup(buf);
+    wictxt->type = Rstrdup(buf);
     if(!IDquiet) {
 	REprintf("Content type '%s'", buf);
 	if(len > 1024*1024)
@@ -1114,6 +1120,7 @@ R_init_internet(DllInfo *info)
     tmp->download = in_do_download;
     tmp->newurl =  in_R_newurl;
     tmp->newsock = in_R_newsock;
+    tmp->newservsock = in_R_newservsock;
 
     tmp->HTTPOpen = in_R_HTTPOpen;
     tmp->HTTPRead = in_R_HTTPRead;
