@@ -268,7 +268,7 @@ removeInputHandler(InputHandler **handlers, InputHandler *it)
 
     if(*handlers == it) {
 	*handlers = (*handlers)->next;
-	free(it);
+	Free(it); // use Free to match allocation with Calloc
 	return(1);
     }
 
@@ -277,7 +277,7 @@ removeInputHandler(InputHandler **handlers, InputHandler *it)
     while(tmp) {
 	if(tmp->next == it) {
 	    tmp->next = it->next;
-	    free(it);
+	    Free(it); // use Free to match allocation with Calloc
 	    return(1);
 	}
 	tmp = tmp->next;
@@ -477,8 +477,10 @@ getSelectedHandler(InputHandler *handlers, fd_set *readMask)
 
 # include <readline/readline.h>
 
-/* For compatibility with pre-readline-4.2 systems, 
-   also missing in Apple's emulation via the NetBSD editline library.*/
+/* For compatibility with pre-readline-4.2 systems, also missing in
+   Apple's emulation via the NetBSD editline library, aka libedit.
+   _RL_FUNCTION_TYPEDEF is not currently defined anywhere.
+*/
 # if !defined (_RL_FUNCTION_TYPEDEF)
 typedef void rl_vcpfunc_t (char *);
 # endif /* _RL_FUNCTION_TYPEDEF */
@@ -496,11 +498,7 @@ typedef void rl_vcpfunc_t (char *);
 attribute_hidden
 char *R_ExpandFileName_readline(const char *s, char *buff)
 {
-#if defined(__APPLE__)
-    char *s2 = tilde_expand_word((char *)s);
-#else
     char *s2 = tilde_expand_word(s);
-#endif
 
     strncpy(buff, s2, PATH_MAX);
     if(strlen(s2) >= PATH_MAX) buff[PATH_MAX-1] = '\0';
@@ -796,7 +794,7 @@ static void initialize_rlcompletion(void)
     /* Tell the completer that we want a crack first. */
     rl_attempted_completion_function = R_custom_completion;
 
-// This was added in readline 6.0
+// This was added in readline 6.0: default is 1 (and was in earlier versions)
 #ifdef HAVE_RL_SORT_COMPLETION_MATCHES
     rl_sort_completion_matches = 0;
 #endif
@@ -891,8 +889,7 @@ static char *R_completion_generator(const char *text, int state)
        completeToken(), and retrieving the completions. */
 
     if (!state) {
-	int i;
-	SEXP completions,
+	SEXP
 	    assignCall = PROTECT(lang2(RComp_assignTokenSym, mkString(text))),
 	    completionCall = PROTECT(lang1(RComp_completeTokenSym)),
 	    retrieveCall = PROTECT(lang1(RComp_retrieveCompsSym));
@@ -900,7 +897,7 @@ static char *R_completion_generator(const char *text, int state)
 
 	eval(assignCall, rcompgen_rho);
 	eval(completionCall, rcompgen_rho);
-	PROTECT(completions = eval(retrieveCall, rcompgen_rho));
+	SEXP completions = PROTECT(eval(retrieveCall, rcompgen_rho));
 	list_index = 0;
 	ncomp = length(completions);
 	if (ncomp > 0) {
@@ -909,8 +906,16 @@ static char *R_completion_generator(const char *text, int state)
 		UNPROTECT(4);
 		return (char *)NULL;
 	    }
-	    for (i = 0; i < ncomp; i++)
-		compstrings[i] = strdup(translateChar(STRING_ELT(completions, i)));
+	    for (int i = 0; i < ncomp; i++) {
+		compstrings[i] =
+		    strdup(translateChar(STRING_ELT(completions, i)));
+		if (!compstrings[i]) {
+		    UNPROTECT(4);
+		    for (int j = 0; j < i; j++) free(compstrings[j]);
+		    free(compstrings);
+		    return (char *)NULL;
+		}
+	    }
 	}
 	UNPROTECT(4);
 	vmaxset(vmax);
@@ -1156,8 +1161,10 @@ void attribute_hidden Rstd_Busy(int which)
    If ask = SA_SUICIDE, no save, no .Last, possibly other things.
  */
 
-
-
+/* Note: now there is R_unlink(), but rm -Rf may be optimized for specific
+   file-systems. Some file systems allow to delete directory trees without
+   explicit/synchronous traversal so that deletion appears to be very
+   fast (see e.g. zfs). */
 void R_CleanTempDir(void)
 {
     char buf[1024];
