@@ -976,6 +976,14 @@ body(cd@contains[["A"]]@coerce)[[2]] ## >>   value <- new("A")
 ## for a few days in R-devel (Nov.2017)
 
 
+## Error messages occurring during method selection are forwarded
+f <- function(x) x
+setGeneric("f")
+setMethod("f", signature("NULL"), function(x) NULL)
+err <- tryCatch(f(stop("this is mentioned")), error = identity)
+stopifnot(identical(err$message, "error in evaluating the argument 'x' in selecting a method for function 'f': this is mentioned"))
+
+
 ## canCoerce(obj, .)  when length(class(obj)) > 1 :
 setOldClass("foo")
 setAs("foo", "A", function(from) new("A", foo=from))
@@ -984,13 +992,14 @@ stopifnot( canCoerce(o3, "A") )
 ## failed in R <= 3.6.1
 
 if(require("Matrix")) withAutoprint({
-    (sci <- names(getClass("integer")@contains))
+    sci <- names(getClass("integer")@contains)
     # These 2 classes have *nothing* to do with Matrix:
     setClass("MyClass")
-    setClassUnion("NumOrMyClass", c("numeric", "MyClass"))
-    (nsci <- names(getClass("integer")@contains))
+    ncl <- "NumOrMyClass"
+    setClassUnion(ncl, c("numeric", "MyClass"))
+    nsci <- names(getClass("integer")@contains)
     ## failed in R <= 3.6.2
-    stopifnot(sci %in% nsci)
+    stopifnot(sci %in% nsci, identical(setdiff(nsci, sci), ncl))
 
     setClassUnion('dMatrixOrMatrix', members = c('dMatrix', 'matrix'))
     ## failed in R <= 3.6.2
